package net.sf.jftp.tools;

import net.sf.jftp.config.Settings;
import net.sf.jftp.util.Log;
import net.sf.jftp.net.SftpVerification;
import net.sf.jftp.JFtp;

import java.util.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import com.sshtools.j2ssh.*;
import com.sshtools.j2ssh.authentication.*;
import com.sshtools.j2ssh.configuration.*;
import com.sshtools.j2ssh.connection.*;
import com.sshtools.j2ssh.io.*;
import com.sshtools.j2ssh.session.*;
import com.sshtools.j2ssh.transport.*; 
import com.sshtools.common.hosts.*;  


public class SshShell extends JFrame implements Runnable {
	
	BufferedOutputStream out;
	BufferedInputStream in, err;
	JTextArea text = new JTextArea(25,80);
	JTextField input = new JTextField();
	long off;
	Thread runner;
	JScrollPane textP;
	int port;
	SshConnectionProperties properties;
	
	public SshShell(SshConnectionProperties properties, String user, String pass) {
		this.port = port;
		this.properties = properties;
		
		try {
			init(user, pass);
		}
		catch (Exception e) {
			e.printStackTrace();
			Log.debug("ERROR: "+e.getMessage());
		}	
	}
	
	public SshShell() {
		try {
			String host = JOptionPane.showInternalInputDialog(
				JFtp.desktop,"Please enter a host:");
			String user = JOptionPane.showInternalInputDialog(
				JFtp.desktop,"Please enter your username:");
			String pass = JOptionPane.showInternalInputDialog(
				JFtp.desktop,"Please enter your password:");

			if(host != null && user != null && pass != null) {
				init(user, pass);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			Log.debug("ERROR: "+e.getMessage());
		}
	}
	
	public void init(String user, String pass) throws Exception {
		
		setTitle("SSH Shell");
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setLocation(50,100);
		
		textP = new JScrollPane(text);
		
		getContentPane().setLayout(new BorderLayout(5,5));
		getContentPane().add("Center", textP);
		getContentPane().add("South", input);
		
		text.setEditable(false);
		text.setLineWrap(true);
		
		input.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_ENTER) {
					send();
				}
			}
		});
			
		ConfigurationLoader.initialize(false);
		
		SshClient ssh = new SshClient();
		ssh.setSocketTimeout(30000);
		//SshConnectionProperties properties = new SshConnectionProperties();
		//properties.setHost(host);
		//properties.setPort(port);		
		//properties.setPrefPublicKey("ssh-dss");
		
		if(Settings.getEnableSshKeys()) {	
 		ssh.connect(properties, new DialogHostKeyVerification(this));
		}
		else {
		 ssh.connect(properties, new SftpVerification(Settings.sshHostKeyVerificationFile));
		}
		
		PasswordAuthenticationClient pwd = new PasswordAuthenticationClient();
		pwd.setUsername(user);
		pwd.setPassword(pass);
		
		int result = ssh.authenticate(pwd);
		
		if (result == AuthenticationProtocolState.COMPLETE) {
			
			SessionChannelClient session = ssh.openSessionChannel();
			if(!session.requestPseudoTerminal("vt100", 80, 24, 0, 0, ""))
				Log.debug("ERROR: Could not open terminal");
						
			if (session.startShell()) {
				out = new BufferedOutputStream(session.getOutputStream());
				in = new BufferedInputStream(session.getInputStream());
				err = new BufferedInputStream(session.getStderrInputStream());
				
				//session.getState().waitForState(ChannelState.CHANNEL_CLOSED);
			} else Log.debug("ERROR: Could not start shell");
		}
		else {
			ssh.disconnect();
		}
								
		pack();
		setVisible(true);
		
		runner = new Thread(this);
		runner.start();

		toFront();
		input.requestFocus();
	}

	public void run() {
		try {
			byte[] b = new byte[1024];
			int i;
		
			while((i = in.read(b)) != StreamTokenizer.TT_EOF) {
				text.append(new String(b,0,i));
				//Log.out("recv: "+i+" -> "+new String(b));
				
				while(err.available() > 0) {
					err.read(b);
					text.append(new String(b,0,i));					
				}
				
				try { Thread.sleep(100); }
				catch(Exception ex) { ex.printStackTrace(); }
				
				JScrollBar bar = textP.getVerticalScrollBar();
				bar.setValue(bar.getMaximum());
			}
		}
		catch(Exception ex) {
			ex.printStackTrace();
			Log.debug("ERROR: "+ex.getMessage());
			this.dispose();
		}
	}
	
	private void send() {
	 try {
		String msg = input.getText()+"\n";
		input.setText("");
		
		out.write(msg.getBytes());
		out.flush();
		
		//Log.out("send: "+msg);		
	 }
	 catch(IOException ex) {
		 ex.printStackTrace();
		 Log.debug("ERROR: "+ex.getMessage());
		 this.dispose();
	 }
	}
	
	
	public static void main(String args[]) {
		new SshShell(); 
	}
	
}

